import array
import constants

from ByteOperations import *
from MonnitTcp import TcpClient
from encryption import encrypt_packet, decrypt_packet
import os
from configparser import ConfigParser
from app_logging import AppLogger


class DLImageRequest:
    # 64000000 00000000 02  00  0000  0000    0100          BD61330B C50E0255361B0100D4D49F020011F70066
    # GWID    Device ID/Image/Length/Position

    uDevID = [0x00, 0x00, 0x00, 0x00] # Device ID
    Security = [0x00, 0x00, 0x00, 0x00]
    Version = [0x04]
    Sequence = [0x00]
    Power = [0x00, 0x00]
    MsgCnt = [0x00, 0x00]
    MsgType = [0xFD, 0x00]
    uImage = [0x00, 0x00, 0x00, 0x00]  # FWID
    uLength = [0x00, 0x00]

    def __init__(self, gwid, args, count):
        self.args = args
        self.uImage = constants.FWID
        self.uLength = constants.uLength
        self.uType = constants.OTAType
        constants.MTCP_Sequence[0] += 1
        constants.MTCP_Sequence[0] %= 256
        self.Sequence = constants.MTCP_Sequence
        # Up message general frame: Identifiers(10) Control(6) Payload(x)
        # self.message = (gwid +  # 4 bytes (I)
        #                 self.Security +  # 4 bytes (I)
        #                 self.Version +  # 1 byte (I)
        #                 self.Sequence +  # 1 byte (this is part of version) (I)
        #                 self.Power +  # 2 bytes (C)
        #                 self.MsgCnt +  # 2 bytes (C)
        #                 self.MsgType +  # 2 bytes (C)
        #                 self.uImage +  # 4 bytes (PL)
        #                 self.uLength +  # 4 bytes (PL)
        #                 self.uFlags)  # 1 byte (PL)
        # if type(count) is int:
        #     count = count.to_bytes(2, byteorder='little', signed=False)
        # self.message += count

    def getMessage(self, gwid):
        self.message = (gwid +  # 4 bytes (I)
                        self.Security +  # 4 bytes (I)
                        self.Version +  # 1 byte (I)
                        self.Sequence +  # 1 byte (this is part of version) (I)
                        self.Power +  # 2 bytes (C)
                        self.MsgType +  # 2 bytes (C)
                        self.MsgCnt +  # 2 bytes (C)
                        self.uImage +  # 4 bytes (PL)
                        # self.uLength +  # 2 bytes (PL)
                        self.uType)  # 1 byte (PL)
        self.message = array.array('B', self.message)
        count = ByteOperations.from_bytes(self.MsgCnt)
        count += 1
        bob = []
        bob += count.to_bytes(2, byteorder='little', signed=False)
        self.MsgCnt = bob
        return list(self.message)

    def GetDLImage(self, gwid):
        message = self.getMessage(gwid)
        message = encrypt_packet(message, self.args)
        encrypted_response = TcpClient.send_data(message, self.args)
        response = decrypt_packet(encrypted_response, self.args)

        int_response = [b for b in response[0:24]]
        count = int.from_bytes(int_response[16:18], byteorder='little')
        cmd = int.from_bytes(int_response[14:16], byteorder='little')
        length = int.from_bytes(int_response[22:24], byteorder='little')
        if count == 0xFFFF:
            constants.BLDL = False
        x = response[24:len(response)]
        return x

    @staticmethod
    def GetCompleteImage(gwid, args):
        # image = []
        filecheck = os.path.isfile('updateImage.txt')
        if filecheck:
            os.remove('updateImage.txt')
        count = []
        constants.BLDL = True
        imagerequest = DLImageRequest(gwid, args, count)
        while True:
            image = imagerequest.GetDLImage(gwid)
            ImageByteArray = bytearray(image)
            with open('updateImage.txt', 'ab') as f:
                f.write(ImageByteArray)
            if not constants.BLDL:
                break
