import array
import json
import constants

from datetime import datetime
from ByteOperations import ByteOperations
from PayloadMessage import PAYLOAD_MESSAGE_TYPE
from app_logging import AppLogger


class GatewayMessage(object):
    # 64000000 00000000 02  00  0000  0000    0100          BD61330B C50E0255361B0100D4D49F020011F70066
    # GWID    Security Ver Seq Power MsgType Count Payload Time     MSG
    # GWID is set statically by the initializing script (from args)
    Security = [0x00, 0x00, 0x00, 0x00]
    Version = [0x04]
    Sequence = [0]
    Power = [0x00, 0x00]
    MsgType = [0x00]
    More = [0x00]

    def __init__(self, messages, args):
        constants.MTCP_Sequence[0] += 1
        constants.MTCP_Sequence[0] %= 256
        self.Sequence = constants.MTCP_Sequence
        self.message = args.gwid + \
                       self.Security + \
                       self.Version + \
                       self.Sequence + \
                       self.Power + \
                       self.MsgType + \
                       self.More
        log = AppLogger()
        count = len(messages)
        countbytes = [count & 0x00FF, (count & 0xFF00) >> 8]
        self.message += countbytes
        self.payload = []
        self.urgent = False
        monnit_epoch = datetime(2010, 1, 1)
        for time_key, message_value in messages.items():
            # Add Message To Payload
            self.options = message_value[2]
            self.urgent = self.urgent or ((self.options & 0x2) != 0)
            self.time = monnit_epoch + time_key
            self.payload += ByteOperations.to_bytes(int(time_key.total_seconds()), 4)
            self.payload += [iv for iv in message_value]
            message_type = message_value[3]

            # Print Json if message type is implemented
            if message_type in PAYLOAD_MESSAGE_TYPE.keys():
                payload_message_class = PAYLOAD_MESSAGE_TYPE[message_type]
                payload_message = payload_message_class(message_value)
                payload_message.write_to_std_out()
            else:
                hex_type = hex(message_type)
                # print(hex_type)
        self.message += self.payload


    def getMessage(self, urgent):
        self.urgent = urgent
        if urgent:
            self.message[13] |= 0x01
        return self.message

    def isUrgent(self):
        return self.urgent

    @classmethod
    def json_encoder(cls, obj):
        if isinstance(obj, datetime):
            return str(obj)
        raise TypeError

    def as_json(self):
        return json.loads(str(self))

    def __str__(self):
        return json.dumps(self.__dict__, default=GatewayMessage.json_encoder)
