import datetime
import serial
import sys
import time

from glob import glob
from ByteOperations import ByteOperations
from CommandGenerator import CommandGenerator
from PayloadMessage import NetworkStatusReport
from SerialCommunication import send_message
from errors import USBException
from app_logging import AppLogger


def get_devices():
    devices = glob('/dev/ttyUSB*')
    return devices


# Returns the location of a working serial device
# https://stackoverflow.com/questions/12090503/listing-available-com-ports-with-python?utm_medium=organic&utm_source=google_rich_qa&utm_campaign=google_rich_qa
def get_usb_location():
    log = AppLogger()
    if sys.platform.startswith('win'):
        ports = ['COM%s' % (i + 1) for i in range(256)]
    elif sys.platform.startswith('linux') or sys.platform.startswith('cygwin'):
        # this excludes your current terminal "/dev/tty"
        ports = glob('/dev/tty[A-Za-z]*')
        # ports = glob('/dev/ttyUSB*')
    elif sys.platform.startswith('darwin'):
        log.info('c')
        ports = glob('/dev/tty.*')
    else:
        log.info('d')
        usb_exception = 'USB SERIAL Platform not supported'
        log.error(usb_exception)
        raise Exception(usb_exception)

    result = []
    for port in ports:
        try:
            s = serial.Serial(port)
            s.close()
            result.append(port)
        except Exception as e:
            if 'Permission denied' in str(e):
                usb_exception = USBException('Permission denied while trying connection to:  ' + port).as_json()
                log.error('{}'.format(usb_exception))
            # usb_exception = USBException(str(e)).as_json()
            # log.error('{}'.format(usb_exception))
            continue
    return result


def get_usb_location_with_gwid(gwid):
    for device in get_devices():
        utc_now = int(get_time().total_seconds())
        time_array = ByteOperations.to_bytes(utc_now, 4)
        with serial.Serial(port=device, baudrate=115200, parity=serial.PARITY_NONE, timeout=.1) as ser:
            active_message = CommandGenerator.getUpdateNetworkStateRequest(1, time_array, 0x00)
            active_response = send_message(ser, active_message)
            if len(active_response) == 0:
                continue
            network_status_report = NetworkStatusReport(active_response)
            device_gwid_bytes = [int(b) for b in network_status_report.apn_serial_id.bytes]
            device_gwid = int.from_bytes(device_gwid_bytes, byteorder='little', signed=False)
            if device_gwid == gwid:
                return device
    return None


def get_time():
    log = AppLogger()
    monnit_epoch = datetime.datetime(2010, 1, 1, 0, 0)
    count = 0
    while monnit_epoch > datetime.datetime.utcnow() and count < 14:
        log.info('Waiting for time to be set...')
        time.sleep(1)
        count += 1

    now = datetime.datetime.utcnow()

    if monnit_epoch > now:
        # Set the time to monnit_epoch or now?
        t = datetime.datetime(2012, 1, 1, 0, 0) - monnit_epoch
        log.warning('Time has not been set!')
    else:
        t = now - monnit_epoch
    return t


if __name__ == '__main__':
    print(get_usb_location())
