import socket
from app_logging import AppLogger


class TcpClient:
    BUFFER_SIZE = 1280

    @staticmethod
    def send_data(data, args):
        log = AppLogger()
        if args.no_network:
            return []
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
            retry = 0
            while True:
                s.settimeout(10)
                s.connect((args.host, int(args.port)))
                s.send(data)
                response = s.recv(TcpClient.BUFFER_SIZE)
                resp_list = [x for x in response]
                if not response:
                    log.info('No response from host: {}, port: {}'.format(args.host, args.port))
                    raise Exception('No response from server')
                if len(resp_list) >= 16 or retry > 3:
                    retry += 1
                    break
        return response

    @staticmethod
    def check_server_status(args):
        log = AppLogger()
        try:
            if args.no_network:
                return True
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.connect((args.host, int(args.port)))
            s.close()
            log.info('Server status OK to host: {}, port: {}'.format(args.host, args.port))
            return True
        except Exception as e:
            # sys.stderr.write(str(e) + '\nFailed to Connect.\n')
            log.info('Failed to Connect to host: {}, port: {}, exception: {}'.format(args.host, args.port, e))
            return False

