import array
import constants

from ByteOperations import *
from MonnitTcp import TcpClient
from encryption import encrypt_packet, decrypt_packet
from configparser import ConfigParser
from confighandler import lock_config_file
from confighandler import unlock_config_file
from app_logging import AppLogger


class SensorListRequest:
    # 64000000 00000000 02  00  0000  0000    0100          BD61330B C50E0255361B0100D4D49F020011F70066
    # GWID    Security Ver Seq Power MsgType Count Payload Time     MSG
    Security = [0x00, 0x00, 0x00, 0x00]
    Version = [0x04]
    Power = [0x00, 0x00]
    MsgType = [0x02, 0x00]
    Sequence = [0]

    def __init__(self, gwid, args, count):
        constants.MTCP_Sequence[0] += 1
        constants.MTCP_Sequence[0] %= 256
        self.Sequence = constants.MTCP_Sequence
        self.args = args
        self.message = (gwid +
                        self.Security +
                        self.Version +
                        self.Sequence +
                        self.Power +
                        self.MsgType)
        if type(count) is int:
            count = count.to_bytes(2, byteorder='little', signed=False)
        self.message += count
        log = AppLogger()
        if constants.VERBOSE_LOGS:
            log.info('SensorListRequest: {}'.format(self.message))

    def getMessage(self):
        self.message = array.array('B', self.message)
        return self.message

    def GetSensorList(self):
        log = AppLogger()
        message = encrypt_packet(self.message, self.args)
        encrypted_response = TcpClient.send_data(message, self.args)
        response = decrypt_packet(encrypted_response, self.args)
        int_response = [b for b in response]
        sensor_list = []
        more_flag = bool(int_response[15])
        count = int.from_bytes(int_response[16:18], byteorder='little')
        i = 18
        try:
            config = ConfigParser()
            lock_config_file('GetSensorList()')
            config.read('config.ini')
            config.set('usb_configs', 'sensors', '{}'.format(count))
            with open('config.ini', 'w') as f:
                config.write(f)
            config.read('config.ini')
        except Exception as ex:
            log.error('File read/write error in GetSensorList: {}'.format(ex))
        finally:
            unlock_config_file('GetSensorList()')
        if constants.VERBOSE_LOGS:
            log.info('Sensor Count: {}'.format(count))
        while i < 18 + count*4:
            sensor_byt = int_response[i:i + 4]
            sensor_list.append(ByteOperations.from_bytes(sensor_byt))
            i += 4
        return sensor_list, more_flag

    @staticmethod
    def GetAllSensors(gwid, arguments):
        sensor_list = []
        count = 0
        while True:
            sensor_list_request = SensorListRequest(gwid, arguments, count)
            sensors, more_flag = sensor_list_request.GetSensorList()
            sensor_list += sensors
            if not more_flag:
                break
            count += len(sensors)
        return sensor_list


# class SensorListReply(object):
#     def __init__(self, byt):
#         self.gwid = int.from_bytes(byt[0:4], byteorder='little')
#         self.security = byt[4:8]
#         self.version = byt[8]
#         self.sequence = byt[9]
#         self.time = int.from_bytes(byt[10:14])
#         self.type = byt[14]
#         self.nothing = byt[15]
#         self.count = byt[16:18]
#         self.body = byt[18:]
