from collections import namedtuple
from Field import TimeField, HexField, BytesField
from MonnitTcp import TcpClient
import constants
import crypto.ecdhe as ecc

# import array
# import json
# from datetime import datetime
# from ByteOperations import ByteOperations
# from CRC8 import CRC8
# from PayloadMessage import PAYLOAD_MESSAGE_TYPE
# from encryption import AESCipher


class StartupMessage(object):


    # 64000000 00000000 02  00  0000  0000    0100          BD61330B C50E0255361B0100D4D49F020011F70066
    # GWID    Security Ver Seq Power MsgType Count Payload Time     MSG
    Security = [0x00, 0x00, 0x00, 0x00]
    Version = [0x04]
    Sequence = [0]
    Power = [0x00, 0x00]
    MsgType = [0x0B]
    More = [0x00]
    Gateway_Type = constants.GATEWAYTYPE
    Gateway_Version = constants.GATEWAYVERSION
    pubkey = (0, 0)

    def __init__(self, args, pk):
        constants.MTCP_Sequence = [0x00]
        self.Sequence = constants.MTCP_Sequence
        self.pubkey = pk
        self.message = args.gwid + \
                       self.Security + \
                       self.Version + \
                       self.Sequence + \
                       self.Power + \
                       self.MsgType + \
                       self.More + \
                       self.Gateway_Type + \
                       self.Gateway_Version
        self.message += list(self.pubkey[0].to_bytes(32, byteorder='little'))
        self.message += list(self.pubkey[1].to_bytes(32, byteorder='little'))

    def as_bytes(self):
        return bytearray(self.message)


class StartupResponse(object):
    def __init__(self, byts, args):
        byts = [int(b) for b in byts]
        self.bytes = BytesField(byts)
        self.gwid = BytesField(byts[:4])
        self.security = BytesField(byts[4:8])
        self.version = HexField(byts[8])
        self.sequence = HexField(byts[9])
        self.time = TimeField(byts[10:14], args)
        self.type = HexField(byts[14])
        self.nothing = HexField(byts[15])
        self.count = int.from_bytes(byts[16:18], byteorder='little', signed=False)

        if len(byts) > 80:
            x = int.from_bytes(bytes(byts[18:50]), byteorder='little', signed=False)
            y = int.from_bytes(bytes(byts[50:82]), byteorder='little', signed=False)
            self.otherpublic = (x, y)
            # diagnostics
            x = int.from_bytes(bytes(byts[82:114]), byteorder='little', signed=False)
            y = int.from_bytes(bytes(byts[114:146]), byteorder='little', signed=False)
            self.ss = (x, y)
            self.aeskey = list(bytes(byts[146:162]))
            self.aesiv = list(bytes(byts[162:178]))
        else:
            self.otherpublic = None

    def __str__(self):
        return 'Bytes: {}\n\tGateway ID: {}, \n\tSecurity: {}, \n\tVersion: {}, \n\tSequence: {}\n\tTime: {}, \n\tType: {}, \n\tNothing: {}, \n\tCount: {}\n\tOther Public Key: {}' \
            .format(self.bytes, self.gwid, self.security, self.version, self.sequence, self.time, self.type,
                    self.nothing, self.count, self.otherpublic)


def startup_sequence(args):
    priv, pub = ecc.make_keypair()
    smsg = StartupMessage(args, pub)
    if constants.server_com is True:
        rsp = TcpClient.send_data(smsg.as_bytes(), args)
        srsp = StartupResponse(rsp, args)
        if srsp.otherpublic is None:
            args.encryption = '0'
            args.generated_key = []
            args.generated_iv = []
        else:
            ss = ecc.compute_sharedsecret(priv, srsp.otherpublic)
            args.generated_key, args.generated_iv = ecc.compute_aes_keyiv(ss, srsp.count)
        return args
    else:
        args.encryption = '0'
        args.generated_key = []
        args.generated_iv = []
        return args


if __name__ == '__main__':

    """OLD METHOD"""
    # gateway_id = 5
    # gwid = [int(i) for i in gateway_id.to_bytes(4, byteorder='little', signed=False)]
    # static_key = bytes(bytearray.fromhex('00000000000000000000000000000001'))
    # arg_tuple = namedtuple('args', ['gwid', 'no_network', 'host', 'port', 'use_local_time', 'encryption'])
    # args = arg_tuple(gwid, False, 't1.sensorsgateway.com', 3000, True, static_key)
    # startup_message = StartupMessage(args)
    #print(startup_message.as_bytes())
    # response = TcpClient.send_data(startup_message.as_bytes(), args)
    #print(response)
    # startup_response = StartupResponse(response, args)
    # args.generated_key = startup_response.generated_key
    # args.generated_iv = startup_response.iv
    # print(startup_response)
