import argparse
from app_logging import AppLogger

from GetUSBLocation import get_usb_location


def get_parser():
    # if constants.VERBOSE_LOGS:
    #     cs.CSClient().log(constants.APP_NAME, 'get_parser() - Enter')
    log = AppLogger()
    parser = argparse.ArgumentParser()
    parser.add_argument('--gwid', help='Specify the ID of the USB Pro to be used')
    parser.add_argument('-i', '--input',
                        help='Specify the interface with the USB Pro (/dev/ttyUSB0, /dev/ttyUSB1, etc)',
                        default=get_usb_location())
    parser.add_argument('--use-local-time', '-t', help='Print local time instead of UTC time',
                        action='store_true')
    parser.add_argument('--pretty', help='Print the messages with pretty printer', action='store_true')
    parser.add_argument('--quiet', '--no-output',
                        help='Run without json output. Messages will still be sent to the specified server.',
                        action='store_true')
    parser.add_argument('--host', '--hostname', help='Specify where to send the sensor data bytes',
                        default='t1.sensorsgateway.com')
    parser.add_argument('--port', help='Specify where to send the sensor data bytes', default=3000)
    parser.add_argument('--no-network', '--local-only', help='Disable sending messages to the host',
                        action='store_true')
    parser.add_argument('--encryption', help='Run with encryption enabled/disabled. Usage Example: \n\t --encryption 1', default=0)
    parser.add_argument('--sleep-time',
                        help='Time to sleep (seconds, int) between querying the gateway', default=5)
    parser.add_argument('--heartbeat',
                        help='Minimum wait time (minutes, int) between delivering data to the server',
                        default=5)
    parser.add_argument('--error-sleep-time',
                        help='Wait time (seconds, int) before retrying after a serious error',
                        default=10)
    parser.add_argument('--max-error-retries',
                        help='Amount of times the script will try to recover from serious errors',
                        default=5)
    parser.add_argument('--no-retry-limit',
                        help='Allows for unlimited retries when errors occur',
                        action='store_true')
    parser.add_argument('--max-up-queue-size',
                        help='Maximum size of queue of messages to be sent to the server. If this size is exceeded, the messages will be sent immediately.',
                        default=100)
    parser.add_argument('--run-once', help='Alternate Run Mode. Retrieve and deliver messages once and exit',
                        action='store_true')
    parser.add_argument('--reform-network',
                        help='Alternate Run Mode. Sends a reform network command to the USB Pro and exits.',
                        action='store_true')
    parser.add_argument('--add-sensors-from-server',
                        help='Alternate Run Mode. Retrieves a list of sensors from the specified server and sends it to the gateway.',
                        action='store_true')
    parser.add_argument('--add-sensors',
                        help='Alternate Run Mode. Tell the USB Pro to listen for the specified comma separated sensor ids. Example: --add-sensors 1,2,3.',
                        default='')
    # if constants.VERBOSE_LOGS:
    #     cs.CSClient().log(constants.APP_NAME, 'get_parser() - End')
    return parser
