import threading
import constants
from app_logging import AppLogger

log = AppLogger()

# Lock for protecting access to the config.ini file. Use
# lock_config_file(who) and unlock_config_file(who) so logs
# will be generated when and who the lock is acquired or
# released. The who parameter will default to 'unknown'.
# The function name could be passed in or anything else
# that would help debugging lock issues.
config_file_lock = threading.RLock()


# Acquire the card file lock and log it.
def lock_config_file(who='unknown'):
    if constants.VERBOSE_LOGS is True:
        log.debug('Acquire config_file_lock for {}'.format(who))
    rc = config_file_lock.acquire()
    if constants.VERBOSE_LOGS is True:
        log.debug('Acquired config_file_lock for {}. rc = {}'.format(who, rc))


# Release the card file lock and log it.
def unlock_config_file(who='unknown'):
    if constants.VERBOSE_LOGS is True:
        log.debug('Release config_file_lock for {}'.format(who))
    config_file_lock.release()
    if constants.VERBOSE_LOGS is True:
        log.debug('Released config_file_lock for {}'.format(who))


def rebuild(config):
    log.info('Rebuilding Config.ini...')
    config['usb_configs'] = {'gwid': '0',
                             'sensors': '0',
                             'reform': '0',
                             'download': '0',
                             'encryption': '1',
                             'heartbeat': '5',
                             'sleep_time': '1',
                             'send': '0',
                             'host': 't1.sensorsgateway.com',
                             'port': '3000',
                             'aware': '1',
                             'state': 'Active'}
    config['state'] = {'usb': 1}

    with open('config.ini', 'w') as f:
        config.write(f)


    # def write(self, config, key, value):
    #     config.set('usb_configs', str(key), str(value))
    #   with open(file, 'w') as f:
    #     config.write(f)

