#!/usr/bin/env python
from app_logging import AppLogger

try:
    import sys
    import traceback
    import array
    from crypto.pyaes.aes import aes128 as aes
    import os

except Exception as e:
    log = AppLogger()
    log.error('Import failure: {}'.format(e))
    log.info('Traceback: {}'.format(traceback.format_exc()))
    sys.exit(-1)


iv_count = None


def encrypt_packet(packet, args):
    if args.encryption == '0':
        return bytes(packet)

    else:
        header_size = 10
        global iv_count

        if iv_count is None:
            iv_count = list(os.urandom(4))
        else:
            icnt = int.from_bytes(iv_count, byteorder='little', signed=False)+2
            if icnt == 0:
                icnt += 2
            iv_count = list(icnt.to_bytes(4, byteorder='little', signed=False))

        # define new header with encryption enumerated
        header = packet[:header_size]
        device_id = header[0:4]
        remaining_header = header[8:10]
        header = device_id + iv_count + remaining_header

        # First Recover the length of the original message
        original_size = list(len(packet).to_bytes(2, byteorder='little', signed=False))
        # Next figure out how much to pad by, if less than 2, then bump to next block
        toadd = 16 - ((len(packet) - header_size) % 16)
        if toadd < 2:
            toadd += 16
        # Add the padding and append length
        initial_padding = list(os.urandom(toadd))
        initial_padding = initial_padding[:-2] + original_size
        # build new body
        body = packet[header_size:]
        body += initial_padding
        # Perform Encryption

        encrypted_body = list(aes(args.generated_key).encrypt_cbc(body, args.generated_iv[:12] + iv_count))
        return bytes(header + encrypted_body)


def decrypt_packet(packet, args):
    header_size = 10
    if args.encryption == '0':
        return packet
    else:
        header = bytes(packet[:header_size])
        iv_helper = header[4:8]

        if iv_helper == b'\x00\x00\x00\x00':
            return packet

        body = packet[header_size:]
        body = bytes(body)
        decrypted_body = aes(args.generated_key).decrypt_cbc(body, args.generated_iv[:12] + list(iv_helper))
        length = int.from_bytes(decrypted_body[-2:], byteorder='little', signed=False)
        decrypted_message = header + decrypted_body
        decrypted_message = decrypted_message[:length]
        return decrypted_message


def test():
    print('test - running ...')


if __name__ == '__main__':
    test()
