'''
This is a driver for the Monnit Gateway. It is a
communication bridge between the iMonnit Server and the
gateway. It allows Monnit USB Gateways to be connected
to the router.
'''
import sys
import argparse
import time
import threading
import os

from confighandler import rebuild
from confighandler import lock_config_file
from confighandler import unlock_config_file
from configparser import ConfigParser
from USB_Pro import start_usb_pro
from start_server import start_server
from app_logging import AppLogger

# usb_pro_thread = Thread(target=start_usb_pro, args=())


def thread_control():
    log = AppLogger()
    pro_stop = threading.Event()
    threads = list()
    server_thread = threading.Thread(target=start_server, args=())
    server_thread.start()
    pro_thread = None
    # determine if config file exists, if not build it with defaults
    i = 0
    config = ConfigParser()
    filecheck = os.path.isfile('config.ini')
    if not filecheck:
        lock_config_file('thread_control() - 1')
        log.warning('Could not find config.ini, rebuilding.')
        rebuild(config)
        unlock_config_file('thread_control() - 1')
    while True:
        if pro_thread is not None:
            alive = pro_thread.is_alive()
            if not alive:
                pro_thread.join()
                threads.remove(pro_thread)
                pro_thread = None
        else:
            try:
                lock_config_file('thread_control() - 2')
                config.read('config.ini')
                config.set('usb_configs', 'download', '1')
                with open('config.ini', 'w') as f:
                    config.write(f)
                config.read('config.ini')
                gwstate = int(config.get('state', 'usb'))
                if (gwstate == 1) or (gwstate == 2):
                    pro_thread = threading.Thread(target=start_usb_pro, args=())
                    threads.append(pro_thread)
                    pro_thread.start()
                    i = i + 1
            except Exception as e:
                log.info('failed to download {}'.format(e))
                rebuild(config)
            finally:
                unlock_config_file('thread_control() - 2')
        timecheck = time.gmtime()
        if timecheck.tm_sec == 0:
            log.info('Main Thread status: Active')
        time.sleep(int(1))


def action(command):
    log = AppLogger()
    if command == 'start':
        log.info('Monnit Driver Started.')
        # Remove the action from the system arguments.
        del sys.argv[1]

        thread_control()
    elif command == 'stop':
        log.info('Monnit Driver Stopped.')


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    log = AppLogger()
    parser.add_argument('opt')
    args = parser.parse_args()

    if args.opt not in ['start', 'stop']:
        log.error('Failed to run command: {}'.format(args.opt))
        exit()

    action(args.opt)
