'''
This is a driver for the Monnit Gateway. It is a
communication bridge between the iMonnit Server and the
gateway. It allows Monnit USB Gateways to be connected
to the router.
'''


import cgi
import time

from http.server import HTTPServer, SimpleHTTPRequestHandler
from http import HTTPStatus
from configparser import ConfigParser
from confighandler import lock_config_file
from confighandler import unlock_config_file
from app_logging import AppLogger


def start_usb():
    log = AppLogger()
    config = ConfigParser()
    try:
        lock_config_file('start_usb()')
        config.read('config.ini')
        config.set('state', 'usb', '1')
        config.set('usb_configs', 'state', 'Active')
        with open('config.ini', 'w') as f:
            config.write(f)
    except Exception as ex:
        log.error('File read/write error in start_usb: {}'.format(ex))
    finally:
        unlock_config_file('start_usb()')


def stop_usb():
    log = AppLogger()
    config = ConfigParser()
    try:
        lock_config_file('stop_usb()')
        config.read('config.ini')
        config.set('state', 'usb', '0')
        config.set('usb_configs', 'state', 'Idle')
        with open('config.ini', 'w') as f:
            config.write(f)
    except Exception as ex:
        log.error('File read/write error in stop_usb: {}'.format(ex))
    finally:
        unlock_config_file('stop_usb()')


def download_sensors():
    log = AppLogger()
    config = ConfigParser()
    try:
        lock_config_file('download_sensors()')
        config.read('config.ini')
        config.set('usb_configs', 'state', 'Busy')
        config.set('state', 'usb', '2')
        with open('config.ini', 'w') as f:
            config.write(f)
    except Exception as ex:
        log.error('File read/write error in download_sensors: {}'.format(ex))
    finally:
        unlock_config_file('download_sensors()')

# def send_reform():
#     constants.BLFlag = True
#     constants.FWID = [0xC5, 0x00, 0x00, 0x00]
#     constants.uLength = [0x54, 0xC1]  # 49492
#     constants.uFlags = [0x0F]


def send_reform():
    log = AppLogger()
    config = ConfigParser()
    try:
        lock_config_file('send_reform()')
        config.read('config.ini')
        config.set('usb_configs', 'reform', '1')
        config.set('usb_configs', 'state', 'Busy')
        config.set('state', 'usb', '2')
        with open('config.ini', 'w') as f:
            config.write(f)
    except Exception as ex:
        log.error('File read/write error in send_reform: {}'.format(ex))
    finally:
        unlock_config_file('send_reform()')


def set_host(value):
    log = AppLogger()
    config = ConfigParser()
    try:
        lock_config_file('set_host()')
        config.read('config.ini')
        config.set('state', 'usb', '2')
        config.set('usb_configs', 'state', 'Busy')
        config.set('usb_configs', 'host', '{}'.format(value))
        with open('config.ini', 'w') as f:
            config.write(f)
        log.info('Host set to : {}'.format(value))
    except Exception as ex:
        log.error('File read/write error in set_host: {}'.format(ex))
    finally:
        unlock_config_file('set_host()')


def set_port(value):
    log = AppLogger()
    config = ConfigParser()
    try:
        lock_config_file('set_port()')
        config.read('config.ini')
        config.set('state', 'usb', '2')
        config.set('usb_configs', 'state', 'Busy')
        config.set('usb_configs', 'port', '{}'.format(value))
        with open('config.ini', 'w') as f:
            config.write(f)
        log.info('Port set to : {}'.format(value))
    except Exception as ex:
        log.error('File read/write error in set_port: {}'.format(ex))
    finally:
        unlock_config_file('set_port()')


def set_heartbeat(value):
    log = AppLogger()
    config = ConfigParser()
    try:
        lock_config_file('set_heartbeat()')
        config.read('config.ini')
        if str.isdecimal(value):
            config.set('state', 'usb', '2')
            config.set('usb_configs', 'state', 'Busy')
            config.set('usb_configs', 'heartbeat', '{}'.format(value))
        else:
            config.set('usb_configs', 'error', '1')
        with open('config.ini', 'w') as f:
            config.write(f)
        log.info('Heartbeat set to (minutes): {}'.format(value))
    except Exception as ex:
        log.error('File read/write error in set_heartbeat: {}'.format(ex))
    finally:
        unlock_config_file('set_heartbeat()')


def set_transaware(value):
    log = AppLogger()
    config = ConfigParser()
    try:
        lock_config_file('set_transaware()')
        config.read('config.ini')
        config.set('state', 'usb', '2')
        config.set('usb_configs', 'state', 'Busy')
        config.set('usb_configs', 'aware', '{}'.format(value))
        with open('config.ini', 'w') as f:
            config.write(f)
        log.info('Transmit set to : {}'.format(value))
    except Exception as ex:
        log.error('File read/write error in set_transaware: {}'.format(ex))
    finally:
        unlock_config_file('set_transaware()')


# def set_encryption(value):
#     log = AppLogger()
#     log.info('Encryption funct')
#     config = ConfigParser()
#     config.read('config.ini')
#     config.set('state', 'usb', '2')
#     config.set('usb_configs', 'heartbeat', '{}'.format(value))
#     constants.USB_Pro = False
#     time.sleep(5)
#     constants.USB_Pro = True
#     log.info('encryption done')


def send_messages():
    log = AppLogger()
    config = ConfigParser()
    try:
        lock_config_file('send_messages()')
        config.read('config.ini')
        config.set('usb_configs', 'state', 'Busy')
        config.set('usb_configs', 'send', '1')
        with open('config.ini', 'w') as f:
            config.write(f)
        log.info('Sending messages to server: {}')
    except Exception as ex:
        log.error('File read/write error in send_messages: {}'.format(ex))
    finally:
        unlock_config_file('send_messages()')


def ip_config(ip_data):
    log = AppLogger()
    log.info('ip_config() - ipdata: {}'.format(ip_data))


def handle_uploaded_config_file(file_data):
    file_data_str = file_data.decode('utf-8')
    log = AppLogger()
    log.info('handle_uploaded_config_file(): {}'.format(file_data_str))


def get_ip_config_info(form):
    ip_key = 'ip_addr'
    subnet_mask_key = 'subnet_mask'
    default_gateway_key = 'default_gateway'
    ip_data = dict()
    ip_data[ip_key] = form.getvalue(ip_key, '')
    ip_data[subnet_mask_key] = form.getvalue(subnet_mask_key, '')
    ip_data[default_gateway_key] = form.getvalue(default_gateway_key, '')
    return ip_data


class WebServerRequestHandler(SimpleHTTPRequestHandler):
    def do_GET(self):
        log = AppLogger()
        joe = time.gmtime()
        if joe.tm_sec == 0:
            log.info('Port 9001 Status: Active')
        config = ConfigParser()
        lock_config_file('do_GET()')
        config.read('config.ini')
        unlock_config_file('do_GET()')
        if self.path == '/status.json':
            gwval = str(config.get('usb_configs', 'gwid'))
            stateval = config.get('usb_configs', 'state')
            sencount = config.get('usb_configs', 'sensors')
            obj = '{' + '"gatewayid":{0},"state":"{1}","devices":{2}'.format(gwval, stateval, sencount) + '}'
            obj = obj.encode('UTF-8')
            self.send_response(200)
            self.send_header("Content-type", "text/html")
            self.send_header("Content-length", len(obj))
            self.end_headers()
            self.wfile.write(obj)
        elif self.path == '/settings.json':
            host = config.get('usb_configs', 'host')
            port = config.get('usb_configs', 'port')
            aware = config.get('usb_configs', 'aware')
            heartbeat = config.get('usb_configs', 'heartbeat')
            obj = '{' + '"host":"{0}","heartbeat":{1},"aware":{3},"port":{2}'.format(host, heartbeat, port, aware) + '}'
            obj = obj.encode('UTF-8')
            lock_config_file('do_GET()')
            config.set('usb_configs', 'error', '0')
            with open('config.ini', 'w') as f:
                config.write(f)
            unlock_config_file('do_GET()')
            self.send_response(200)
            self.send_header("Content-type", "text/html")
            self.send_header("Content-length", len(obj))
            self.end_headers()
            self.wfile.write(obj)
        else:
            super().do_GET()

    def do_POST(self):
        # Log the Get request
        log = AppLogger()
        log.info('Received Post request: {}'.format(self.path))
        # Parse the form data posted
        form = cgi.FieldStorage(
            fp=self.rfile,
            headers=self.headers,
            environ={'REQUEST_METHOD': 'POST',
                     'CONTENT_TYPE': self.headers['Content-Type'],
                     })
        wasaware = 0
        settingspost = 0
        errormsg = ''
        for field in form.keys():
            field_item = form[field]
            # log.info('field: {}'.format(field))
            if field_item.filename:  # The field contains an uploaded file
                file_data = field_item.file.read()
                handle_uploaded_config_file(file_data)
                value = "file {}".format(field_item.filename)
                # log.info('Received Post request value: {}'.format(value))
            else:
                value = form[field].value
                name = form[field].name
                # log.info('Received Post request value: {} Name: {}'.format(value, name))
                if name == 'command':
                    if value == 'SendMessages':
                        send_messages()
                    if value == 'SensorList':
                        download_sensors()
                    if value == 'Reform':
                        send_reform()
                    if value == 'Stop':
                        stop_usb()
                    if value == 'Start':
                        start_usb()

                    self.send_response(HTTPStatus.OK)
                    self.send_header('Content-type', 'text/html')
                    self.end_headers()
                    self.wfile.write(bytes('Success', 'utf-8'))
                    return

                else:  # Check for the value/name that you want to handle.
                    settingspost = 1
                    # if name == 'hostName':
                    #     set_host(value)
                    # if name == 'port':
                    #     set_port(value)
                    if name == 'heartbeat':
                        if str.isdecimal(value):
                            set_heartbeat(value)
                        else:
                            # errormsg = '{' + '"error":{0}'.format(value) + '}'
                            errormsg = 'Invalid Heartbeat'
                    if name == 'transmitAware':
                        wasaware = 1
                    # if name == 'encryptionKey':
                    #     log.info('crypto {}'.format(value))
        if settingspost == 1:
            set_transaware(wasaware)
        errormsg = errormsg.encode('UTF-8')
        self.send_response(200)
        self.send_header("Content-type", "text/html")
        self.send_header("Content-length", len(errormsg))
        self.end_headers()
        self.wfile.write(errormsg)


def start_server():
    # avoid 8080, as the router may have service on it.
    # Firewall rules will need to be changed in the router
    # to allow access on this port.
    # global arguments
    # parser = get_parser()
    # arguments = parser.parse_args()
    
    server_address = ('', 9001)
    log = AppLogger()
    log.info('Starting Server: {}'.format(server_address))
    httpd = HTTPServer(server_address, WebServerRequestHandler)

    try:
        httpd.serve_forever()

    except KeyboardInterrupt:
        log.warning('Stopping Server, Keyboard interrupt')

    return 0
